/*
 * Copyright (c) 2005 - 2009, Tranql project contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.tranql.connector.oracle;

import java.sql.SQLException;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import oracle.jdbc.pool.OracleDataSource;
import org.tranql.connector.jdbc.AbstractLocalDataSourceMCF;

/**
 * ManagedConnectionFactory for connecting to a Oracle RAC enabled database with local (non-XA) transactions.
 *
 * @version $Revision: 626 $ $Date: 2009-03-03 11:42:07 -0800 (Tue, 03 Mar 2009) $
 */
public class LocalRACMCF extends AbstractLocalDataSourceMCF {
    private static final long serialVersionUID = 8672506697045238199L;
    private static final Log log = LogFactory.getLog(LocalMCF.class.getName());
    private final OracleDataSource ds;
    private String password;

    /**
     * Default constructor for an Oracle RAC Local DataSource.
     *
     * @throws SQLException if there was a problem instantiating an OracleDataSource
     */
    public LocalRACMCF() throws SQLException {
        super(new OracleDataSource(), new OracleRACExceptionSorter(), true);
        ds = (OracleDataSource) super.dataSource;
        // the following property has to be set for the ONS.jar, even though its not used in thin mode
        System.setProperty("oracle.ons.oraclehome", ".");
    }

    public String getDatabaseName() {
        return ds.getDatabaseName();
    }

    public String getServiceName() {
        return ds.getServiceName();
    }

    public void setDataSourceName(String dsname) {
	if (log.isDebugEnabled()) {
	    log.debug("setDataSourceName(" + dsname + ")");
	}
        ds.setDataSourceName(dsname);
    }

    public String getDataSourceName() {
        return ds.getDataSourceName();
    }

    public String getDescription() {
        return ds.getDescription();
    }

    public String getNetworkProtocol() {
        return ds.getNetworkProtocol();
    }

    public void setPassword(String pwd) {
        this.password = pwd;
	if (log.isDebugEnabled()) {
	    log.debug("setPassword(????????)");
	}
        ds.setPassword(pwd);
    }

    public String getPassword() {
        return password;
    }

    public Integer getPortNumber() {
        return new Integer(ds.getPortNumber());
    }

    public String getServerName() {
        return ds.getServerName();
    }

    public void setUserName(String user) {
	if (log.isDebugEnabled()) {
	    log.debug("setUserName(" + user + ")");
	}
        ds.setUser(user);
    }

    public String getUserName() {
        return ds.getUser();
    }

    public String getDriverType() {
        return ds.getDriverType();
    }

    public String getTNSEntryName() {
        return ds.getTNSEntryName();
    }

    public String getURL() {
        try {
            return ds.getURL();
        } catch (SQLException e) {
            return "<unknown location>";
        }
    }
    
    public void setURL(String s) {
        if (log.isDebugEnabled()) {
            log.debug("setURL(" + s + ")");
        }
        ds.setURL(s);
    }

    public Integer getMaxStatements() throws SQLException {
        return new Integer(ds.getMaxStatements());
    }

    public void setMaxStatements(Integer ms) throws SQLException {
        int i = ms == null ? 0 : ms.intValue();
	if (log.isDebugEnabled()) {
	    log.debug("setMaxStatements(" + i + ")");
	}
        ds.setMaxStatements(i);
        ds.setImplicitCachingEnabled(i != 0);
    }

    // extra 3 config-property values for Oracle RAC
    public String getConnectionCacheName() throws SQLException {
        return ds.getConnectionCacheName();
    }
    
    public void setConnectionCacheName(String s) throws SQLException {
	if (log.isDebugEnabled()) {
	    log.debug("setConnectionCacheName(" + s + ")");
	}
	    // the following have to be set together in this exact order
	    ds.setConnectionCachingEnabled(true);
        ds.setFastConnectionFailoverEnabled(true);
        ds.setConnectionCacheName(s);
    }
    
    public String getONSConfiguration() throws SQLException {
        return ds.getONSConfiguration();
    }
    
    public void setONSConfiguration(String s) throws SQLException {
	if (log.isDebugEnabled()) {
	    log.debug("setONSConfiguration(" + s + ")");
	}
        ds.setONSConfiguration(s);
    }
    
    public Properties getConnectionCacheProperties() throws SQLException {
        return ds.getConnectionCacheProperties();
    }
    
    public void setConnectionCacheProperties(String s) throws SQLException {
        if (log.isDebugEnabled()) {
            log.debug("setConnectionCacheProperties(" + s + ")");
        }
        // Should look something like the below, with any number of spaces around the "," or "=" -
        //   "InitialLimit=5,MinLimit=5,MaxLimit=10"
        //   "InitialLimit=5, MinLimit=5, MaxLimit=10"
        //   "InitialLimit = 5 , MinLimit = 5 , MaxLimit = 10"
        Properties props = new Properties();
        String []args = s.trim().split("[,]");
        for (int i=0; i<args.length; i++) {
            String []prop = args[i].trim().split("[ =]+");
            if (prop.length == 2) {
                props.setProperty(prop[0], prop[1]);
            } else {
                log.error("setConnectionCacheProperties(" + s + ")");
                log.error("Found invalid Oracle ConnectionCacheProperties(" + prop.toString() + ")");
            }
        }

        if (log.isDebugEnabled()) {
            log.debug("Setting ConnectionCacheProperties=" + props.toString());
        }
        // Setup ConnectionCaching Properties
        ds.setConnectionCacheProperties(props);
    }
        
    /**
     * Equality is defined in terms of the serverName, portNumber and databaseName properties
     *
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj instanceof LocalRACMCF) {
            try {
                OracleDataSource other = ((LocalRACMCF) obj).ds;
                return equals(ds.getURL(), other.getURL()) &&
                       equals(ds.getConnectionCacheName(), other.getConnectionCacheName());
            } catch (SQLException e) {
                return false;
            }
        } else {
            return false;
        }
    }

    private static boolean equals(String a, String b) {
        return a == b || a != null && a.equals(b);
    }

    public int hashCode() {
        try {
            return hashCode(ds.getURL()) ^ hashCode(ds.getConnectionCacheName());
        } catch (SQLException e) {
            return 0;
        }
    }

    private static int hashCode(String s) {
        return s == null ? 0 : s.hashCode();
    }

    public String toString() {
        return "LocalRACMCF[" + getURL() + "]";
    }
}
